unit dBaseReport;

interface

uses
  { System }
  System.SysUtils,
  System.Classes,
  { ReportBuilder }
  ppParameter,
  ppDesignLayer,
  ppBands,
  ppCache,
  ppClass,
  ppComm,
  ppRelatv,
  ppProd,
  ppReport,
  ppPrnabl,
  ppCtrls,
  { Uniform }
  uSettings;

type
  TdmBaseReport = class(TDataModule)
    ppReport: TppReport;
    ppHeaderBand1: TppHeaderBand;
    ppDetailBand1: TppDetailBand;
    ppFooterBand1: TppFooterBand;
    ppDesignLayers1: TppDesignLayers;
    ppDesignLayer1: TppDesignLayer;
    ppParameterList1: TppParameterList;
    ppTitle: TppLabel;
    AppImage: TppImage;
  private
    FReportNumber: Integer;
    FSettings: TSettings;
    function GenerateReport_To_Archive: TMemoryStream;
    function GenerateReport_To_PDF: TMemoryStream;
    procedure LoadLogo;
  protected
    FChartName: string;
    procedure AfterInitializeReport;
    function DateToStrChartAxis(const ADate:TDateTime): string;
    procedure InitializeReport; virtual; abstract;
  public
    property Settings: TSettings read FSettings write FSettings;
    property Report: TppReport read ppReport;
    function GenerateReport(AReportNumber: Integer): TMemoryStream;
  end;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

uses
  { ReportBuilder }
  ppFilDev,
  ppPDFDevice,
  ppTypes;

{ TdmBaseReport }

procedure TdmBaseReport.AfterInitializeReport;
begin
  if (FChartName <> '') then
    ppTitle.Text := FChartName + ' - ' + IntToStr(FReportNumber)
  else
    ppTitle.Text := 'Report ' + IntToStr(FReportNumber);
end;

function TdmBaseReport.DateToStrChartAxis(const ADate: TDateTime): string;
begin
  Result := Format('%s%s%s', [FormatDateTime('DD-MM', ADate),
                              sLineBreak,
                              FormatDateTime('YYYY', ADate)]);
end;

function TdmBaseReport.GenerateReport(AReportNumber: Integer): TMemoryStream;
begin
  FReportNumber := AReportNumber;
  if FSettings.DoPrintDirectlyToPdf then
    Result := GenerateReport_To_PDF
  else
    Result := GenerateReport_To_Archive;
end;

function TdmBaseReport.GenerateReport_To_Archive: TMemoryStream;
var
  LDevice: TppFileDevice;
  LReportStream: TMemoryStream;
begin
  Result := TMemoryStream.Create;
  ppReport.DeviceType := dtArchive;
  LDevice := TppArchiveDevice.Create(nil);
  try
    LoadLogo;
    InitializeReport;
    AfterInitializeReport;
    LReportStream := TMemoryStream.Create;
    try
      LDevice.OutputStream := LReportStream;
      LDevice.Publisher := ppReport.Publisher;
      ppReport.PrintToDevices;
      LReportStream.Position := 0;
      Result.CopyFrom(LReportStream, LReportStream.Size);
    finally
      LReportStream.Free;
    end;
  finally
    LDevice.Free;
  end;
end;

function TdmBaseReport.GenerateReport_To_PDF: TMemoryStream;
var
  LDevice: TppFileDevice;
  LReportStream: TMemoryStream;
begin
  Result := TMemoryStream.Create; 
  ppReport.DeviceType := dtPDF;          
  LDevice := TppPDFDevice.Create(nil);
  try                                   
    LDevice.Publisher := ppReport.Publisher;      
    LDevice.Encoding := TEncoding.UTF8;
    LDevice.IncludeBOM := True;
    LoadLogo;
    InitializeReport;
    AfterInitializeReport;
    ppReport.Device := dvFile;
    LReportStream := TMemoryStream.Create;
    try
      LDevice.OutputStream := LReportStream;
      ppReport.PrintToDevices;
      LReportStream.Position := 0;
      Result.CopyFrom(LReportStream, LReportStream.Size);
    finally
      LReportStream.Free;
    end;
  finally
    LDevice.Free;
  end;
end;

procedure TdmBaseReport.LoadLogo;
begin
  if FSettings.DoLoadLogo then
  begin
    AppImage.Width := Trunc(AppImage.Width);
    AppImage.Picture.LoadFromFile('Logo\Logo.png');
  end;
end;

end.
