unit fMain;

interface

uses
  { System }
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.ExtCtrls,
  Vcl.StdCtrls,
  Vcl.Samples.Spin,
  { Uniform }
  uSettings,
  uWorkerManager;

const
  LOG_MAP = 'C:\Temp\ReportBuilder';

type
  TfrmMain = class(TForm)
    mmText: TMemo;
    pnlTop: TPanel;
    btnStart: TButton;
    btnStop: TButton;
    lblNumberOfThreads: TLabel;
    edtNumberOfThreads: TSpinEdit;
    lblNumberOfReports: TLabel;
    edtNumberOfReports: TSpinEdit;
    chckbxLogReports: TCheckBox;
    lblGenerateChart: TLabel;
    cbxChart: TComboBox;
    lblMaxPauze: TLabel;
    edtMaxPause: TSpinEdit;
    chckbxPrintDirectlyToPdf: TCheckBox;
    chckbxMergeReports: TCheckBox;
    chckbxLoadLogo: TCheckBox;
    procedure btnStartClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure chckbxPrintDirectlyToPdfClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FSettings: TSettings;
    FWorkerManager: TWorkerManager;
    procedure DoFinish;
    procedure FillSettings;
    function HasRunningThreads: Boolean;
  public
    destructor Destroy; override;
  end;

var
  frmMain: TfrmMain;

implementation

uses
  { Uniform }
  uLogManager;

{$R *.dfm}

procedure TfrmMain.btnStartClick(Sender: TObject);
var
  LLogFolderReports: string;
begin
  LLogFolderReports := '';
  if chckbxLogReports.Checked then
    LLogFolderReports := LOG_MAP;
  if HasRunningThreads then
  begin
    DoFinish;
  end;
  mmText.Lines.Clear;
  try
    btnStart.Enabled := False;
    btnStop.Enabled := True;
    if not Assigned(FWorkerManager) then
      FWorkerManager := TWorkerManager.Create;
    LogManager.MemoLog(mmText.Lines);
    LogManager.Initialize(LOG_MAP);
    FillSettings;
    FWorkerManager.Execute(FSettings,
      procedure
      begin
        btnStart.Enabled := True;
        btnStop.Enabled := False;
      end);
  except
    on E: Exception do
    begin
      mmText.Lines.Add('ERROR: ' + E.Message);
    end;
  end;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  DoFinish;
end;

procedure TfrmMain.chckbxPrintDirectlyToPdfClick(Sender: TObject);
begin
  chckbxMergeReports.Enabled := not chckbxPrintDirectlyToPdf.Checked;
  if chckbxPrintDirectlyToPdf.Checked then
    chckbxMergeReports.checked := False;
end;

destructor TfrmMain.Destroy;
begin
  LogManager.MemoLog(nil);
  DoFinish;
  if Assigned(FWorkerManager) then
    FWorkerManager.Free;
  FSettings.Free;
  inherited Destroy;
end;

procedure TfrmMain.DoFinish;
begin
  if Assigned(FWorkerManager) then
  begin
    FWorkerManager.Stop;
  end;
end;

procedure TfrmMain.FillSettings;
var
  LLogFolderReports: string;
begin
  LLogFolderReports := '';
  if chckbxLogReports.Checked then
    LLogFolderReports := LOG_MAP;
  FSettings.Free;
  FSettings := TSettings.Create;
  FSettings.Chart := cbxChart.ItemIndex;
  FSettings.DoLoadLogo := chckbxLoadLogo.Checked;
  FSettings.DoLogReports := chckbxLogReports.Checked;
  FSettings.DoMergeReports := chckbxMergeReports.Checked;
  FSettings.DoPrintDirectlyToPdf := chckbxPrintDirectlyToPdf.Checked;
  FSettings.LogFolder := LLogFolderReports;
  FSettings.NumberOfReports := edtNumberOfReports.Value;
  FSettings.NumberOfThreads := edtNumberOfThreads.Value;
  FSettings.Pause := edtMaxPause.Value;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  ReportMemoryLeaksOnShutdown  := True;
end;

function TfrmMain.HasRunningThreads: Boolean;
begin
  Result := False;
  if Assigned(FWorkerManager) then
  begin
    Result := (FWorkerManager.CountRunningThreads > 0);
  end;
end;

end.
