unit uThread;

interface

uses
  { System }
  System.Classes,
  { Uniform }
  uGlobalWorker,
  uSettings;

type
  TWorkerThread = class(TThread)
  private
    FNumber: Integer;
    FSettings: TSettings;
    FWorker: IGlobalWorker;
  protected
    procedure Execute; override;
    procedure RandomPause;
  public
    property Number: Integer read FNumber;
    property Worker: IGlobalWorker read FWorker write FWorker;
    constructor Create(ASettings: TSettings; ANumber: Integer);
    procedure Finalize;
  end;

implementation

uses
  { System }
  System.SysUtils,
  { Uniform }
  uLogManager;

{ TWorkerThread }

constructor TWorkerThread.Create(ASettings: TSettings; ANumber: Integer);
begin
  inherited Create(True);
  FNumber := ANumber;
  FSettings := ASettings;
end;

procedure TWorkerThread.Execute;
var
  LNumber: Integer;
begin
  inherited;
  LogManager.LogMessageInfo('Start thread ' + IntToStr(FNumber));
  for LNumber := 1 to FSettings.NumberOfReports do
  begin
    if not Terminated then
    begin
      FWorker.ExecuteReport(LNumber);
      RandomPause;
    end;
  end;
end;

procedure TWorkerThread.Finalize;
begin
  FWorker.Finalize;
end;

procedure TWorkerThread.RandomPause;
var
  LPause: Integer;
begin
  if (FSettings.Pause > 0) then
  begin
    LPause := Random(FSettings.Pause);
    if (LPause > 0) then
      Sleep(LPause);
  end;
end;

end.
